#   ************    LibreSilicon's StdCellLibrary   *******************
#
#   Organisation:   Chipforge
#                   Germany / European Union
#
#   Profile:        Chipforge focus on fine System-on-Chip Cores in
#                   Verilog HDL Code which are easy understandable and
#                   adjustable. For further information see
#                           www.chipforge.org
#                   there are projects from small cores up to PCBs, too.
#
#   File:           StdCellLib/Catalog/stacked2_cells.mk
#
#   Purpose:        Makefile for Cell Generation with popcorn
#
#   ************    GNU Make 3.80 Source Code       ****************
#
#   ////////////////////////////////////////////////////////////////
#
#   Copyright (c)   2018, 2019 by
#                   chipforge - <popcorn@nospam.chipforge.org>
#   All rights reserved.
#
#       This Standard Cell Library is licensed under the Libre Silicon
#       public license; you can redistribute it and/or modify it under
#       the terms of the Libre Silicon public license as published by
#       the Libre Silicon alliance, either version 1 of the License, or
#       (at your option) any later version.
#
#       This design is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#       See the Libre Silicon Public License for more details.
#
#   ////////////////////////////////////////////////////////////////////

#   ----------------------------------------------------------------
#               DESCRIPTION
#   ----------------------------------------------------------------

#   list all dependencies for cells with up to 2 stacked transistors

#   ----------------------------------------------------------------
#               CELL TARGETS
#   ----------------------------------------------------------------

ifeq ($(BUFFER),2)

#   --------    now buffered    ------------------------------------

CELLS +=        AAO22.cell \
                AND2.cell \
                AO21.cell \
                OA21.cell \
                OOA22.cell \
                OR2.cell

AAO22.cell:          DESCR = "2-2-input AND-AND-OR gate"
AAO22.cell:          LEVEL = 2
AAO22.cell:          AO21.cell
	$(POPCORN) -m nand -c $@ $< > $@

AND2.cell:           DESCR = "2-input AND gate"
AND2.cell:           INV.cell
	$(POPCORN) -m nand -c $@ $< > $@

AO21.cell:           DESCR = "2-1-input AND-OR gate"
AO21.cell:           OR2.cell
	$(POPCORN) -m aoi -c $@ $< > $@

OA21.cell:           DESCR = "2-1-input OR-AND gate"
OA21.cell:           AND2.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA22.cell:          DESCR = "2-2-input OR-OR-AND gate"
OOA22.cell:          LEVEL = 2
OOA22.cell:          OA21.cell
	$(POPCORN) -m nor -c $@ $< > $@

OR2.cell:            DESCR = "2-input OR gate"
OR2.cell:            INV.cell
	$(POPCORN) -m nor -c $@ $< > $@

BUFFERED = true

else

#   --------    not buffered    ------------------------------------

CELLS +=        AAOI22.cell \
                AOI21.cell \
                NAND2.cell \
                NOR2.cell \
                OAI21.cell \
                OOAI22.cell

AAOI22.cell:         DESCR = "2-2-input AND-AND-OR-Invert gate"
AAOI22.cell:         LEVEL = 2
AAOI22.cell:         AOI21.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOI21.cell:          DESCR = "2-1-input AND-OR-Invert gate"
AOI21.cell:          NOR2.cell
	$(POPCORN) -m aoi -c $@ $< > $@

NAND2.cell:          DESCR = "2-input Not-AND (or NAND) gate"
NAND2.cell:          INV.cell
	$(POPCORN) -m nand -c $@ $< > $@

NOR2.cell:           DESCR = "2-input Not-OR (or NOR) gate"
NOR2.cell:           LEVEL = 2
NOR2.cell:           INV.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAI21.cell:          DESCR = "2-1-input OR-AND-Invert gate"
OAI21.cell:          NAND2.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOAI22.cell:         DESCR = "2-2-input OR-OR-AND-Invert gate"
OOAI22.cell:         LEVEL = 2
OOAI22.cell:         OAI21.cell
	$(POPCORN) -m nor -c $@ $< > $@

endif
