#   ************    LibreSilicon's StdCellLibrary   *******************
#
#   Organisation:   Chipforge
#                   Germany / European Union
#
#   Profile:        Chipforge focus on fine System-on-Chip Cores in
#                   Verilog HDL Code which are easy understandable and
#                   adjustable. For further information see
#                           www.chipforge.org
#                   there are projects from small cores up to PCBs, too.
#
#   File:           StdCellLib/Catalog/stacked3_cells.mk
#
#   Purpose:        Makefile for Cell Generation with popcorn
#
#   ************    GNU Make 3.80 Source Code       ****************
#
#   ////////////////////////////////////////////////////////////////
#
#   Copyright (c)   2018, 2019 by
#                   chipforge - <popcorn@nospam.chipforge.org>
#   All rights reserved.
#
#       This Standard Cell Library is licensed under the Libre Silicon
#       public license; you can redistribute it and/or modify it under
#       the terms of the Libre Silicon public license as published by
#       the Libre Silicon alliance, either version 1 of the License, or
#       (at your option) any later version.
#
#       This design is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#       See the Libre Silicon Public License for more details.
#
#   ////////////////////////////////////////////////////////////////////

#   ----------------------------------------------------------------
#               DESCRIPTION
#   ----------------------------------------------------------------

#   list all dependencies for cells with 3 stacked transistors
#
include stacked2_cells.mk

#   ----------------------------------------------------------------
#               CELL TARGETS
#   ----------------------------------------------------------------

ifdef BUFFERED

#   --------    already buffered    --------------------------------

CELLS +=        AAAO332.cell \
                AAAO333.cell \
                AAO331.cell \
                AND3.cell \
                AO22.cell \
                AO31.cell \
                AO311.cell \
                AAO32.cell \
                AAO33.cell \
                AO331.cell \
                AO332.cell \
                AOA211.cell \
                AOA221.cell \
                AOAO2111.cell \
                AOOA212.cell \
                OA22.cell \
                OA31.cell \
                OA32.cell \
                OAO211.cell \
                OAO212.cell \
                OAO221.cell \
                OAOA2111.cell \
                OR3.cell \
                OOA32.cell \
                OOA321.cell \
                OOA33.cell \
                OOA331.cell \
                OOOA332.cell \
                OOOA333.cell

AAAO332.cell:        DESCR = "3-3-2-input AND-AND-AND-OR gate"
AAAO332.cell:        LEVEL = 3
AAAO332.cell:        AAAO331.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAAO333.cell:        DESCR = "3-3-3-input AND-AND-AND-OR gate"
AAAO333.cell:        LEVEL = 3
AAAO333.cell:        AAAO332.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAO331.cell:         DESCR = "3-3-1-input AND-AND-OR gate"
AAO331.cell:         LEVEL = 3
AAO331.cell:         AAO321.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AND3.cell:           DESCR = "3-input AND gate"
AND3.cell:           AND2.cell
	$(POPCORN) -m nand -c $(*F) -c $@ $< > $@

AO22.cell:           DESCR = "2-2-input AND-OR gate"
AO22.cell:           OR3.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AO31.cell:           DESCR = "3-1-input AND-OR gate"
AO31.cell:           AO21.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAO32.cell:          DESCR = "3-2-input AND-AND-OR gate"
AAO32.cell:          AAO22.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AO321.cell:          DESCR = "3-2-1-input AND-OR gate"
AO321.cell:          AO221.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAO33.cell:          DESCR = "3-3-input AND-AND-OR gate"
AAO33.cell:          LEVEL = 3
AAO33.cell:          AAO32.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOA211.cell:         DESCR = "2-1-1-input AND-OR-AND gate"
AOA211.cell:         OA21.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOA221.cell:         DESCR = "2-2-1-input AND-OR-AND gate"
AOA221.cell:         OA31.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOAO2111.cell:       DESCR = "2-1-1-1-input AND-OR-AND-OR gate"
AOAO2111.cell:       OAO211.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOOA212.cell:        DESCR = "2-1-2-input AND-OR-OR-AND gate"
AOOA212.cell:        OOA22.cell
	$(POPCORN) -m nand -c $@ $< > $@

OA22.cell:           DESCR = "2-2-input OR-AND gate"
OA22.cell:           AND3.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA31.cell:           DESCR = "3-1-input OR-AND gate"
OA31.cell:           OA21.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA32.cell:           DESCR = "3-2-input OR-AND gate"
OA32.cell:           OA22.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA32.cell:          DESCR = "3-2-input OR-OR-AND gate"
OOA32.cell:          OOA22.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAO211.cell:         DESCR = "2-1-1-input OR-AND-OR gate"
OAO211.cell:         AO21.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAO212.cell:         DESCR = "2-1-2-input OR-AND-OR gate"
OAO212.cell:         AO22.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAO221.cell:         DESCR = "2-2-1-input OR-AND-OR gate"
OAO221.cell:         AO31.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOA2111.cell:       DESCR = "2-1-1-1-input OR-AND-OR-AND gate"
OAOA2111.cell:       AOA211.cell
	$(POPCORN) -m nor -c $@ $< > $@

OR3.cell:            DESCR = "3-input OR gate"
OR3.cell:            OR2.cell
	$(POPCORN) -m nor -c $@ $< > $@

OOA321.cell:         DESCR = "3-2-1-input OR-OR-AND gate"
OOA321.cell:         OOA221.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA33.cell:          DESCR = "3-3-input OR-OR-AND gate"
OOA33.cell:          LEVEL = 3
OOA33.cell:          OOA32.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA331.cell:         DESCR = "3-3-1-input OR-OR-AND gate"
OOA331.cell:         LEVEL = 3
OOA331.cell:         OOA321.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOOA332.cell:        DESCR = "3-3-2-input OR-OR-OR-AND gate"
OOOA332.cell:        LEVEL = 3
OOOA332.cell:        OOA331.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOOA333.cell:        DESCR = "3-3-3-input OR-OR-OR-AND gate"
OOOA333.cell:        LEVEL = 3
OOOA333.cell:        OOOA332.cell
	$(POPCORN) -m oai -c $@ $< > $@

else
ifeq ($(BUFFER),3)

#   --------    now buffered    ------------------------------------

CELLS +=        AAAO332.cell \
                AAAO333.cell \
                AAO331.cell \
                AND3.cell \
                AO31.cell \
                AO311.cell \
                AAO32.cell \
                AO33.cell \
                AO331.cell \
                AO332.cell \
                AOA211.cell \
                AOA221.cell \
                AOAO2111.cell \
                AOOA212.cell \
                OA22.cell \
                OA31.cell \
                OA32.cell \
                OAO211.cell \
                OAO212.cell \
                OAO221.cell \
                OAOA2111.cell \
                OR3.cell \
                OOA32.cell \
                OOA321.cell \
                OOA33.cell \
                OOA331.cell \
                OOOA332.cell \
                OOOA333.cell

AAAO332.cell:        DESCR = "3-3-2-input AND-AND-AND-OR gate"
AAAO332.cell:        LEVEL = 3
AAAO332.cell:        AAAO331.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAAO333.cell:        DESCR = "3-3-3-input AND-AND-AND-OR gate"
AAAO333.cell:        LEVEL = 3
AAAO333.cell:        AAAO332.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAO331.cell:         DESCR = "3-3-1-input AND-AND-OR gate"
AAO331.cell:         LEVEL = 3
AAO331.cell:         AAO321.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AND3.cell:           DESCR = "3-input AND gate"
AND3.cell:           NAND2.cell
	$(POPCORN) -m nand -c $@ $< > $@

AO22.cell:           DESCR = "2-2-input AND-OR gate"
AO22.cell:           OR3.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AO31.cell:           DESCR = "3-1-input AND-OR gate"
AO31.cell:           AOI21.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAO32.cell:          DESCR = "3-2-input AND-AND-OR gate"
AAO32.cell:          AAOI22.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AO321.cell:          DESCR = "3-2-1-input AND-OR gate"
AO321.cell:          AO221.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAO33.cell:          DESCR = "3-3-input AND-AND-OR gate"
AAO33.cell:          LEVEL = 3
AAO33.cell:          AAO32.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOA211.cell:         DESCR = "2-1-1-input AND-OR-AND gate"
AOA211.cell:         OAI21.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOA221.cell:         DESCR = "2-2-1-input AND-OR-AND gate"
AOA221.cell:         OA31.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOAO2111.cell:       DESCR = "2-1-1-1-input AND-OR-AND-OR gate"
AOAO2111.cell:       OAO211.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOOA212.cell:        DESCR = "2-1-2-input AND-OR-OR-AND gate"
AOOA212.cell:        OOAI22.cell
	$(POPCORN) -m nand -c $@ $< > $@

OA22.cell:           DESCR = "2-2-input OR-AND gate"
OA22.cell:           AND3.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA31.cell:           DESCR = "3-1-input OR-AND gate"
OA31.cell:           OAI21.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA32.cell:           DESCR = "3-2-input OR-AND gate"
OA32.cell:           OA22.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA32.cell:          DESCR = "3-2-input OR-OR-AND gate"
OOA32.cell:          OOAI22.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAO211.cell:         DESCR = "2-1-1-input OR-AND-OR gate"
OAO211.cell:         AOI21.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAO212.cell:         DESCR = "2-1-2-input OR-AND-OR gate"
OAO212.cell:         AOI22.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAO221.cell:         DESCR = "2-2-1-input OR-AND-OR gate"
OAO221.cell:         AO31.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOA2111.cell:       DESCR = "2-1-1-1-input OR-AND-OR-AND gate"
OAOA2111.cell:       AOA211.cell
	$(POPCORN) -m nor -c $@ $< > $@

OR3.cell:            DESCR = "3-input OR gate"
OR3.cell:            NOR2.cell
	$(POPCORN) -m nor -c $@ $< > $@

OOA321.cell:         DESCR = "3-2-1-input OR-OR-AND gate"
OOA321.cell:         OOA221.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA33.cell:          DESCR = "3-3-input OR-OR-AND gate"
OOA33.cell:          LEVEL = 3
OOA33.cell:          OOA32.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOA331.cell:         DESCR = "3-3-1-input OR-OR-AND gate"
OOA331.cell:         LEVEL = 3
OOA331.cell:         OOA321.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOOA332.cell:        DESCR = "3-3-2-input OR-OR-OR-AND gate"
OOOA332.cell:        LEVEL = 3
OOOA332.cell:        OOA331.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOOA333.cell:        DESCR = "3-3-3-input OR-OR-OR-AND gate"
OOOA333.cell:        LEVEL = 3
OOOA333.cell:        OOOA332.cell
	$(POPCORN) -m oai -c $@ $< > $@

BUFFERED = true

else

#   --------    not buffered    ------------------------------------

CELLS +=        AAAOI332.cell \
                AAAOI333.cell \
                AAOI321.cell \
                AAOI331.cell \
                AOAI211.cell \
                AOAI221.cell \
                AOAOI2111.cell \
                AOI211.cell \
                AOI31.cell \
                AAOI32.cell \
                AAOI33.cell \
                AOOAI212.cell \
                NAND3.cell \
                NOR3.cell \
                OAI22.cell \
                OAI31.cell \
                OAI32.cell \
                OAOAI2111.cell \
                OAOI211.cell \
                OAOI212.cell \
                OAOI221.cell \
                OOAI32.cell \
                OOAI321.cell \
                OOAI33.cell \
                OOAI331.cell \
                OOOAI332.cell \
                OOOOAI333.cell

AAAOI332.cell:       DESCR = "3-3-2-input AND-AND-AND-OR-Invert gate"
AAAOI332.cell:       LEVEL = 3
AAAOI332.cell:       AAOI331.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAAOI333.cell:       DESCR = "3-3-3-input AND-AND-AND-OR-Invert gate"
AAAOI333.cell:       LEVEL = 3
AAAOI333.cell:       AAAOI332.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAOI331.cell:        DESCR = "3-3-1-input AND-AND-OR-Invert gate"
AAOI331.cell:        LEVEL = 3
AAOI331.cell:        AAOI321.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOAI211.cell:        DESCR = "2-1-1-input AND-OR-AND-Invert gate"
AOAI211.cell:        OAI21.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOAI221.cell:        DESCR = "2-2-1-input AND-OR-AND-Invert gate"
AOAI221.cell:        OAI31.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOAOI2111.cell:      DESCR = "2-1-1-1-input AND-OR-AND-OR-Invert gate"
AOAOI2111.cell:      OAOI211.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOI22.cell:          DESCR = "2-2-input AND-OR-Invert gate"
AOI22.cell:          NOR3.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOI31.cell:          DESCR = "3-1-input AND-OR-Invert gate"
AOI31.cell:          AOI21.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAOI32.cell:         DESCR = "3-2-input AND-AND-OR-Invert gate"
AAOI32.cell:         AAOI22.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAOI321.cell:        DESCR = "3-2-1-input AND-AND-OR-Invert gate"
AAOI321.cell:        AAOI221.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AAOI33.cell:         DESCR = "3-3-input AND-AND-OR-Invert gate"
AAOI33.cell:         LEVEL = 3
AAOI33.cell:         AAOI32.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOOAI212.cell:       DESCR = "2-1-2-input AND-OR-OR-AND-Invert gate"
AOOAI212.cell:       OOAI22.cell
	$(POPCORN) -m nand -c $@ $< > $@

NAND3.cell:          DESCR = "3-input Not-AND (or NAND) gate"
NAND3.cell:          NAND2.cell
	$(POPCORN) -m nand -c $@ $< > $@

NOR3.cell:           DESCR = "3-input Not-OR (or NOR) gate"
NOR3.cell:           NOR2.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAI22.cell:          DESCR = "2-2-input OR-AND-Invert gate"
OAI22.cell:          NAND3.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAI31.cell:          DESCR = "3-1-input OR-AND-Invert gate"
OAI31.cell:          OAI21.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAI32.cell:          DESCR = "3-2-input OR-AND-Invert gate"
OAI32.cell:          OAI22.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOAI32.cell:         DESCR = "3-2-input OR-OR-AND-Invert gate"
OOAI32.cell:         OOAI22.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAOAI2111.cell:      DESCR = "2-1-1-1-input OR-AND-OR-AND-Invert gate"
OAOAI2111.cell:      AOAI211.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOI211.cell:        DESCR = "2-1-1-input OR-AND-OR-Invert gate"
OAOI211.cell:        AOI21.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOI212.cell:        DESCR = "2-1-2-input OR-AND-OR-Invert gate"
OAOI212.cell:        AOI22.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOI221.cell:        DESCR = "2-2-1-input OR-AND-OR-Invert gate"
OAOI221.cell:        AOI31.cell
	$(POPCORN) -m nor -c $@ $< > $@

OOAI321.cell:        DESCR = "3-2-1-input OR-OR-AND-Invert gate"
OOAI321.cell:        OOAI221.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOAI33.cell:         DESCR = "3-3-input OR-OR-AND-Invert gate"
OOAI33.cell:         LEVEL = 3
OOAI33.cell:         OOAI32.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOAI331.cell:        DESCR = "3-3-1-input OR-OR-AND-Invert gate"
OOAI331.cell:        LEVEL = 3
OOAI331.cell:        OOAI321.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOOAI332.cell:       DESCR = "3-3-2-input OR-OR-OR-AND-Invert gate"
OOOAI332.cell:       LEVEL = 3
OOOAI332.cell:       OOAI331.cell
	$(POPCORN) -m oai -c $@ $< > $@

OOOAI333.cell:       DESCR = "3-3-3-input OR-OR-OR-AND-Invert gate"
OOOAI333.cell:       LEVEL = 3
OOOAI333.cell:       OOOAI332.cell
	$(POPCORN) -m oai -c $@ $< > $@

endif
endif
